#	Copyright (C) 2017 Battelle Memorial Institute
# file: matpower_dict.py
# tested on case9_9Subst.m
# it's assumed that whenever a section starts, eg mpc.buses, 
#    1) actual data begins on the next line
#    2) only one object entry per line
# further, only 2nd-order polynomial gencost models are parsed
import json;
import sys;

def matpower_dict (nameroot):
	ip = open (nameroot + ".m", "r")
	op = open (nameroot + "_m_dict.json", "w")

	baseMVA = 1
	ampFactor = 1
	fncsBuses = {}
	subBuses = {}
	# generators have their bus number
	generators = {}
	# gencosts have 1-1 correspondence to generators, but no direct input of bus number
	gencosts = {}
	idxGen = 1 # used as a key in Python, not an array index
	buses = {}
	inSubNameFNCS = False
	inBusFNCS = False
	inBus = False
	inGen = False
	inGencost = False

	for line in ip:
		lst = line.split()
		if inGencost == True and len(lst) > 6:
			startup = float(lst[1].strip(" ").strip(";").strip("]"))
			shutdown = float(lst[2].strip(" ").strip(";").strip("]"))
			c2 = float(lst[4].strip(" ").strip(";").strip("]"))
			c1 = float(lst[5].strip(" ").strip(";").strip("]"))
			c0 = float(lst[6].strip(" ").strip(";").strip("]"))
			gencosts[idxGen] = {'StartupCost':startup,'ShutdownCost':shutdown,'c2':c2,'c1':c1,'c0':c0}
			idxGen += 1
		if inGen == True and len(lst) > 8:
			bus = int(lst[0].strip(" ").strip(";").strip("]"))
			pnom = float(lst[1].strip(" ").strip(";").strip("]"))
			pmax = float(lst[8].strip(" ").strip(";").strip("]"))
			genfuel = "gas"
			gentype = "combinedcycle"
			generators[idxGen] = {'bus':bus,'bustype':'pv','Pnom':pnom,'Pmax':pmax,'StartupCost':0,'ShutdownCost':0,'c2':0,'c1':0,'c0':0,'genfuel':genfuel,'gentype':gentype}
			idxGen += 1
		if inBus == True and len(lst) > 3:
			bus = int(lst[0].strip(" ").strip(";").strip("]"))
			bustype = int(lst[1].strip(" ").strip(";").strip("]"))
			if bustype == 1:
				bustypename = 'pq'
			elif bustype == 2:
				bustypename = 'pv'
			elif bustype == 3:
				bustypename = 'swing'
			else:
				bustypename = 'unknown'
			pnom = float(lst[2].strip(" ").strip(";").strip("]"))
			qnom = float(lst[3].strip(" ").strip(";").strip("]"))
			busarea = float(lst[6].strip(" ").strip(";").strip("]"))
			buszone = float(lst[10].strip(" ").strip(";").strip("]"))
			buses[bus] = {'bustype':bustypename,'Pnom':pnom,'Qnom':qnom,'area':busarea,'zone':buszone}
		if inSubNameFNCS == True and len(lst) > 1:
			name = lst[0].strip(" ").strip(";").strip("]")
			bus = int(lst[1].strip(" ").strip(";").strip("]"))
			subBuses[name] = bus;
		if inBusFNCS == True:
			bus = int(lst[0].strip(" ").strip(";").strip("]"))
			fncsBuses[bus] = {'Pnom':0,'Qnom':0,'GLDsubstations':[]}
		if len(lst) > 2:
			if lst[0] == "mpc.SubNameFNCS":
				inSubNameFNCS = True
			if lst[0] == "mpc.BusFNCS":
				inBusFNCS = True
			if lst[0] == "mpc.bus":
				inBus = True
			if lst[0] == "mpc.gen":
				inGen = True
				idxGen = 1
			if lst[0] == "mpc.gencost":
				inGencost = True
				idxGen = 1
			if lst[0] == "mpc.baseMVA":
				baseMVA = float(lst[2].strip(" ").strip(";")) * 1.0
			if lst[0] == "mpc.ampFactor":
				ampFactor = float(lst[2].strip(" ").strip(";")) * 1.0
		if line.find("];") >= 0:
			inSubNameFNCS = False
			inBusFNCS = False
			inBus = False
			inGen = False
			inGencost = False

	# backfill the generators with gencosts and bustypes
	for key, val in generators.items():
		cost = gencosts[key]
		row = buses[key]
		val['bustype'] = row['bustype']
		val['StartupCost'] = cost['StartupCost']
		val['ShutdownCost'] = cost['ShutdownCost']
		val['c2'] = cost['c2']
		val['c1'] = cost['c1']
		val['c0'] = cost['c0']

	# backfill the fncsBuses with nominal loads and GridLAB-D substation names
	for key, val in fncsBuses.items():
		row = buses[key]
		val['Pnom'] = row['Pnom']
		val['Qnom'] = row['Qnom']
		val['area'] = row['area']
		val['zone'] = row['zone']
	for key, val in subBuses.items():
		(fncsBuses[val]['GLDsubstations']).append(key)

	matpower = {'baseMVA':baseMVA,'ampFactor':ampFactor,'fncsBuses':fncsBuses,'generators':generators}
	print (json.dumps(matpower), file=op)

	ip.close()
	op.close()



#	Copyright (C) 2017 Battelle Memorial Institute
from TransmissionMetricsProcessor import TransmissionMetricsProcessor
import os
import matplotlib.pyplot as plt

# root = []
# for root, dirs, files in os.walk(r'''C:\Qiuhua\FY2016_Project_Transactive_system\Simulation_Year1\SGIP1\\'''):
#     # print(root)
#     print (dirs)
#     # print (files)

d=r'''C:\Qiuhua\FY2016_Project_Transactive_system\Simulation_Year1\SGIP1'''
subdir = [os.path.join(d,o) for o in os.listdir(d) if os.path.isdir(os.path.join(d,o))]
print(subdir)

foldernames = [name for name in os.listdir(d) if os.path.isdir(os.path.join(d,name))]
print(foldernames)

all_bus_metrics = {}
all_gen_metrics = {}
colors = {0:'k', 1: 'b', 2:'r', 3: 'g',4: 'm',5:'y'}

fig1,ax1 = plt.subplots()

for i in range(0,len(subdir)):
    casepath = subdir[i]+"\\"
    casename = foldernames[i]  # 'SGIP1a'
    tmp = TransmissionMetricsProcessor()

    tmp.loadAllMetricsFromJSONFiles(casename, casepath)

    #print(tmp.get_bus_metrics())
    print('\n\n', casename)
    # print('\n',tmp.get_bus_metrics_at_bus(7).LMP_P.values)
    # print('\n', tmp.get_bus_metrics_at_bus(7).VMAG.values)

    tmp.get_bus_metrics_at_bus(7).LMP_P.plot(ax = ax1, color = colors[i],label=casename)
    #tmp.get_bus_metrics_at_bus(7).VMAG.plot(ax=axes[i])
    all_bus_metrics[casename] = tmp.get_bus_metrics()
    all_gen_metrics[casename] = tmp.get_gen_metrics()

ax1.set(title ='Bus LMP comparision', xlabel='Time (hour)',ylabel='LMP ($/kWh)')


# fig2 = plt.figure(2)





##/// data for table
print('\ntotal gen cost, revenue, profit:')
for case in foldernames:
    earning = all_gen_metrics[case].REVENUE.sum().values
    cost = all_gen_metrics[case].COST.sum().values
    profit = earning-cost
    print(case, ",", cost,",",earning,",",profit)



print('\ntotal gen emissions CO2, Sox, Nox (lb):')
for case in foldernames:
    co2 =  all_gen_metrics[case].EMISSION_CO2.sum().values
    sox = all_gen_metrics[case].EMISSION_SOX.sum().values
    nox = all_gen_metrics[case].EMISSION_NOX.sum().values
    print(case, ",", co2, ",", sox, ",", nox)


fig2, axes = plt.subplots(nrows=len(foldernames))
print('\ntotal Ancillary generation output:')
i = 0
for case in foldernames:
    all_gen_metrics[case].sel(busNum = 1).PGEN.plot( ax = axes[i],color = colors[i],label=casename)
    i = i+1

plt.show()


# Copyright (C) 2017-2020 Battelle Memorial Institute
# file: setup.py

from setuptools import setup, find_packages


setup(
    name='tesp_support',
    version='0.6.4',
    author='Thomas McDermott',
    author_email='Thomas.McDermott@PNNL.gov',
    description='Python support for the Transactive Energy Simulation Platform',
    long_description='\n\n'.join(
        open(f, 'rb').read().decode('utf-8')
        for f in ['README.rst', 'CHANGELOG.rst']),
    url='https://github.com/pnnl/tesp',
    license='BSD',
    install_requires=[
        'pandas>=0.25.1',
        'numpy>=1.17.2',
        'scipy>=1.3.1',
        'matplotlib>=3.0.0',
        'networkx>=2.4',
        'PYPOWER>=5.1.4'
    ],
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering',
    ],
    zip_safe=False
)



# Copyright (C) 2017-2019 Battelle Memorial Institute
# file: process_gld.py
'''Functions to plot data from GridLAB-D

Public Functions:
    :process_gld: Reads the data and metadata, then makes the plots.  

'''
import json;
import sys;
import numpy as np;
try:
  import matplotlib as mpl;
  import matplotlib.pyplot as plt;
except:
  pass

def process_gld(nameroot, dictname = ''):
  ''' Plots a summary/sample of power, air temperature and voltage

  This function reads *substation_nameroot_metrics.json*,  
  *billing_meter_nameroot_metrics.json* and
  *house_nameroot_metrics.json* for the data;
  it reads *nameroot_glm_dict.json* for the metadata.  
  These must all exist in the current working directory.  
  Makes one graph with 4 subplots:
  
  1. Substation real power and losses
  2. Average air temperature over all houses
  3. Min/Max line-to-neutral voltage and Min/Max line-to-line voltage at the first billing meter
  4. Min, Max and Average air temperature at the first house 

  Args:
    nameroot (str): name of the TESP case, not necessarily the same as the GLM case, without the extension
    dictname (str): metafile name (with json extension) for a different GLM dictionary, if it's not *nameroot_glm_dict.json*. Defaults to empty.
  '''

  # the feederGenerator now inserts metrics_collector objects on capacitors and regulators
  bCollectedRegCapMetrics = True 

  # first, read and print a dictionary of all the monitored GridLAB-D objects
  if len (dictname) > 0:
      lp = open (dictname).read()
  else:
      lp = open (nameroot + '_glm_dict.json').read()
  dict = json.loads(lp)
  fdr_keys = list(dict['feeders'].keys())
  fdr_keys.sort()
  inv_keys = list(dict['inverters'].keys())
  inv_keys.sort()
  hse_keys = list(dict['houses'].keys())
  hse_keys.sort()
  mtr_keys = list(dict['billingmeters'].keys())
  mtr_keys.sort()
  cap_keys = list(dict['capacitors'].keys())
  cap_keys.sort()
  reg_keys = list(dict['regulators'].keys())
  reg_keys.sort()
  #print ('Capacitor Keys', cap_keys)
  #print ('Regulator Keys', reg_keys)
  xfMVA = dict['transformer_MVA']
  bulkBus = dict['bulkpower_bus']
#  print('\nBilling Meter Dictionary:')
#  for key in mtr_keys:
#    row = dict['billingmeters'][key]
#    print (key, 'on phase', row['phases'], 'of', row['feeder_id'], 'with', row['children'])
#  print('\nHouse Dictionary:')
#  for key in hse_keys:
#    row = dict['houses'][key]
#    print (key, 'on', row['billingmeter_id'], 'has', row['sqft'], 'sqft', row['cooling'], 'cooling', row['heating'], 'heating', row['wh_gallons'], 'gal WH')
#  print('\nInverter Dictionary:')
#  for key in inv_keys:
#    row = dict['inverters'][key]
#    print (key, 'on', row['billingmeter_id'], 'has', row['rated_W'], 'W', row['resource'], 'resource')

  # parse the substation metrics file first; there should just be one entity per time sample
  # each metrics file should have matching time points
  lp_s = open ('substation_' + nameroot + '_metrics.json').read()
  lst_s = json.loads(lp_s)
  print ('\nMetrics data starting', lst_s['StartTime'])

  # make a sorted list of the sample times in hours
  lst_s.pop('StartTime')
  meta_s = lst_s.pop('Metadata')
  times = list(map(int,list(lst_s.keys())))
  times.sort()
  print ('There are', len (times), 'sample times at', times[1] - times[0], 'second intervals')
  hrs = np.array(times, dtype=np.float)
  denom = 3600.0
  hrs /= denom

  time_key = str(times[0])

  # find the actual substation name (not a feeder name) as GridLAB-D wrote it to the metrics file
  sub_key = list(lst_s[time_key].keys())[0]
  print ('\n\nFile', nameroot, 'has substation', sub_key, 'at bulk system bus', bulkBus, 'with', xfMVA, 'MVA transformer')
  print('\nFeeder Dictionary:')
  for key in fdr_keys:
    row = dict['feeders'][key]
    print (key, 'has', row['house_count'], 'houses and', row['inverter_count'], 'inverters')

  # parse the substation metadata for 2 things of specific interest
#  print ('\nSubstation Metadata for', len(lst_s[time_key]), 'objects')
  for key, val in meta_s.items():
#    print (key, val['index'], val['units'])
    if key == 'real_power_avg':
      SUB_POWER_IDX = val['index']
      SUB_POWER_UNITS = val['units']
    elif key == 'real_power_losses_avg':
      SUB_LOSSES_IDX = val['index']
      SUB_LOSSES_UNITS = val['units']

  # create a NumPy array of all metrics for the substation
  data_s = np.empty(shape=(1, len(times), len(lst_s[time_key][sub_key])), dtype=np.float)
  print ('\nConstructed', data_s.shape, 'NumPy array for Substations')
  j = 0
  for key in [sub_key]:
    i = 0
    for t in times:
      ary = lst_s[str(t)][key]
      data_s[j, i,:] = ary
      i = i + 1
    j = j + 1

  # display some averages
  print ('Maximum power =', 
         '{:.3f}'.format (data_s[0,:,SUB_POWER_IDX].max()), SUB_POWER_UNITS)
  print ('Average power =', 
         '{:.3f}'.format (data_s[0,:,SUB_POWER_IDX].mean()), SUB_POWER_UNITS)
  print ('Average losses =', 
         '{:.3f}'.format (data_s[0,:,SUB_LOSSES_IDX].mean()), SUB_LOSSES_UNITS)

  # read the other JSON files; their times (hrs) should be the same
  lp_h = open ('house_' + nameroot + '_metrics.json').read()
  lst_h = json.loads(lp_h)
  lp_m = open ('billing_meter_' + nameroot + '_metrics.json').read()
  lst_m = json.loads(lp_m)
  lp_i = open ('inverter_' + nameroot + '_metrics.json').read()
  lst_i = json.loads(lp_i)

  # houses
  lst_h.pop('StartTime')
  meta_h = lst_h.pop('Metadata')
#  print('\nHouse Metadata for', len(lst_h[time_key]), 'objects')
  for key, val in meta_h.items():
#    print (key, val['index'], val['units'])
    if key == 'air_temperature_max':
      HSE_AIR_MAX_IDX = val['index']
      HSE_AIR_MAX_UNITS = val['units']
    elif key == 'air_temperature_min':
      HSE_AIR_MIN_IDX = val['index']
      HSE_AIR_MIN_UNITS = val['units']
    elif key == 'air_temperature_avg':
      HSE_AIR_AVG_IDX = val['index']
      HSE_AIR_AVG_UNITS = val['units']
    elif key == 'air_temperature_deviation_cooling':
      HSE_AIR_DEVC_IDX = val['index']
      HSE_AIR_DEVC_UNITS = val['units']
    elif key == 'air_temperature_deviation_heating':
      HSE_AIR_DEVH_IDX = val['index']
      HSE_AIR_DEVH_UNITS = val['units']
    elif key == 'total_load_avg':
      HSE_TOTAL_AVG_IDX = val['index']
      HSE_TOTAL_AVG_UNITS = val['units']
    elif key == 'hvac_load_avg':
      HSE_HVAC_AVG_IDX = val['index']
      HSE_HVAC_AVG_UNITS = val['units']
    elif key == 'waterheater_load_avg':
      HSE_WH_AVG_IDX = val['index']
      HSE_WH_AVG_UNITS = val['units']
  data_h = np.empty(shape=(len(hse_keys), len(times), len(lst_h[time_key][hse_keys[0]])), dtype=np.float)
  print ('\nConstructed', data_h.shape, 'NumPy array for Houses')
  j = 0
  for key in hse_keys:
    i = 0
    for t in times:
      ary = lst_h[str(t)][hse_keys[j]]
      data_h[j, i,:] = ary
      i = i + 1
    j = j + 1

  print ('average all house temperatures Noon-8 pm first day:',
         '{:.3f}'.format (data_h[:,144:240,HSE_AIR_AVG_IDX].mean()))

  # Billing Meters 
  lst_m.pop('StartTime')
  meta_m = lst_m.pop('Metadata')
  nBillingMeters = 0
  if not lst_m[time_key] is None:
    nBillingMeters = len(lst_m[time_key])
#  print('\nBilling Meter Metadata for', nBillingMeters, 'objects')
  for key, val in meta_m.items():
#    print (key, val['index'], val['units'])
    if key == 'voltage_max':
      MTR_VOLT_MAX_IDX = val['index']
      MTR_VOLT_MAX_UNITS = val['units']
    elif key == 'voltage_min':
      MTR_VOLT_MIN_IDX = val['index']
      MTR_VOLT_MIN_UNITS = val['units']
    elif key == 'voltage_avg':
      MTR_VOLT_AVG_IDX = val['index']
      MTR_VOLT_AVG_UNITS = val['units']
    elif key == 'voltage12_max':
      MTR_VOLT12_MAX_IDX = val['index']
      MTR_VOLT12_MAX_UNITS = val['units']
    elif key == 'voltage12_min':
      MTR_VOLT12_MIN_IDX = val['index']
      MTR_VOLT12_MIN_UNITS = val['units']
    elif key == 'voltage_unbalance_max':
      MTR_VOLTUNB_MAX_IDX = val['index']
      MTR_VOLTUNB_MAX_UNITS = val['units']
    elif key == 'bill':
      MTR_BILL_IDX = val['index']
      MTR_BILL_UNITS = val['units']
    elif key == 'above_RangeA_Count':
      MTR_AHI_COUNT_IDX = val['index']
    elif key == 'above_RangeB_Count':
      MTR_BHI_COUNT_IDX = val['index']
    elif key == 'below_RangeA_Count':
      MTR_ALO_COUNT_IDX = val['index']
    elif key == 'below_RangeB_Count':
      MTR_BLO_COUNT_IDX = val['index']
    elif key == 'below_10_percent_NormVol_Count':
      MTR_OUT_COUNT_IDX = val['index']
    elif key == 'above_RangeA_Duration':
      MTR_AHI_DURATION_IDX = val['index']
    elif key == 'above_RangeB_Duration':
      MTR_BHI_DURATION_IDX = val['index']
    elif key == 'below_RangeA_Duration':
      MTR_ALO_DURATION_IDX = val['index']
    elif key == 'below_RangeB_Duration':
      MTR_BLO_DURATION_IDX = val['index']
    elif key == 'below_10_percent_NormVol_Duration':
      MTR_OUT_DURATION_IDX = val['index']

  if nBillingMeters > 0:
    data_m = np.empty(shape=(len(mtr_keys), len(times), len(lst_m[time_key][mtr_keys[0]])), dtype=np.float)
    print ('\nConstructed', data_m.shape, 'NumPy array for Meters')
    j = 0
    for key in mtr_keys:
      i = 0
      for t in times:
        ary = lst_m[str(t)][mtr_keys[j]]
        data_m[j, i,:] = ary
        i = i + 1
      j = j + 1

  # normalize the meter voltages to 100 percent
  j = 0
  for key in mtr_keys:
    vln = dict['billingmeters'][key]['vln'] / 100.0
    vll = dict['billingmeters'][key]['vll'] / 100.0
    data_m[j,:,MTR_VOLT_MIN_IDX] /= vln
    data_m[j,:,MTR_VOLT_MAX_IDX] /= vln
    data_m[j,:,MTR_VOLT_AVG_IDX] /= vln
    data_m[j,:,MTR_VOLT12_MIN_IDX] /= vll
    data_m[j,:,MTR_VOLT12_MAX_IDX] /= vll
    j = j + 1

  lst_i.pop('StartTime')
  meta_i = lst_i.pop('Metadata')
  # assemble the total solar and battery inverter power
  solar_kw = np.zeros(len(times), dtype=np.float)
  battery_kw = np.zeros(len(times), dtype=np.float)
#  print ('\nInverter Metadata for', len(inv_keys), 'objects')
  for key, val in meta_i.items():
#    print (key, val['index'], val['units'])
    if key == 'real_power_avg':
      INV_P_AVG_IDX = val['index']
      INV_P_AVG_UNITS = val['units']
    elif key == 'reactive_power_avg':
      INV_Q_AVG_IDX = val['index']
      INV_Q_AVG_UNITS = val['units']
  if len(inv_keys) > 0:
    data_i = np.empty(shape=(len(inv_keys), len(times), len(lst_i[time_key][inv_keys[0]])), dtype=np.float)
    print ('\nConstructed', data_i.shape, 'NumPy array for Inverters')
    j = 0
    for key in inv_keys:
      i = 0
      for t in times:
        ary = lst_i[str(t)][inv_keys[j]]
        data_i[j, i,:] = ary
        i = i + 1
      j = j + 1
    j = 0
    for key in inv_keys:
      res = dict['inverters'][key]['resource']
      if res == 'solar':
        solar_kw += 0.001 * data_i[j,:,INV_P_AVG_IDX]
      elif res == 'battery':
        battery_kw += 0.001 * data_i[j,:,INV_P_AVG_IDX]
      j = j + 1

  # capacitors and regulators
  lp_c = open ('capacitor_' + nameroot + '_metrics.json').read()
  lst_c = json.loads(lp_c)
  lp_r = open ('regulator_' + nameroot + '_metrics.json').read()
  lst_r = json.loads(lp_r)

  lst_c.pop('StartTime')
  meta_c = lst_c.pop('Metadata')
#  print('\nCapacitor Metadata for', len(cap_keys), 'objects')
  for key, val in meta_c.items():
    if key == 'operation_count':
      CAP_COUNT_IDX = val['index']
      CAP_COUNT_UNITS = val['units']
  if len(cap_keys) > 0 and bCollectedRegCapMetrics:
    data_c = np.empty(shape=(len(cap_keys), len(times), len(lst_c[time_key][cap_keys[0]])), dtype=np.float)
    print ('\nConstructed', data_c.shape, 'NumPy array for Capacitors')
    j = 0
    for key in cap_keys:
      i = 0
      for t in times:
        ary = lst_c[str(t)][cap_keys[j]]
        data_c[j, i,:] = ary
        i = i + 1
      j = j + 1
    print ('Total cap switchings =', data_c[:,-1,CAP_COUNT_IDX].sum())

  lst_r.pop('StartTime')
  meta_r = lst_r.pop('Metadata')
#  print('\nRegulator Metadata for', len(reg_keys), 'objects')
  for key, val in meta_r.items():
    if key == 'operation_count':
      REG_COUNT_IDX = val['index']
      REG_COUNT_UNITS = val['units']
  if len(reg_keys) > 0 and bCollectedRegCapMetrics:
    data_r = np.empty(shape=(len(reg_keys), len(times), len(lst_r[time_key][reg_keys[0]])), dtype=np.float)
    print ('\nConstructed', data_r.shape, 'NumPy array for Regulators')
    j = 0
    for key in reg_keys:
      i = 0
      for t in times:
        ary = lst_r[str(t)][reg_keys[j]]
        data_r[j, i,:] = ary
        i = i + 1
      j = j + 1
    print ('Total tap changes =', data_r[:,-1,REG_COUNT_IDX].sum())

  print ('Total meter bill =', 
         '{:.3f}'.format (data_m[:,-1,MTR_BILL_IDX].sum()))

  # display a plot
  fig, ax = plt.subplots(2, 5, sharex = 'col')

  total1 = (data_h[:,:,HSE_TOTAL_AVG_IDX]).squeeze()
  total2 = total1.sum(axis=0)
  hvac1 = (data_h[:,:,HSE_HVAC_AVG_IDX]).squeeze()
  hvac2 = hvac1.sum(axis=0)
  wh1 = (data_h[:,:,HSE_WH_AVG_IDX]).squeeze()
  wh2 = wh1.sum(axis=0)
  ax[0,0].plot(hrs, 0.001 * data_s[0,:,SUB_POWER_IDX], color='blue', label='Total')
  ax[0,0].plot(hrs, 0.001 * data_s[0,:,SUB_LOSSES_IDX], color='red', label='Losses')
  ax[0,0].plot(hrs, total2, color='green', label='Houses')
  ax[0,0].plot(hrs, hvac2, color='magenta', label='HVAC')
  ax[0,0].plot(hrs, wh2, color='orange', label='WH')
  ax[0,0].set_ylabel('kW')
  ax[0,0].set_title ('Substation Real Power at ' + sub_key)
  ax[0,0].legend(loc='best')

  #vabase = dict['inverters'][inv_keys[0]]['rated_W']
  #print ('Inverter base power =', vabase)
  #ax[0,1].plot(hrs, data_i[0,:,INV_P_AVG_IDX] / vabase, color='blue', label='Real')
  #ax[0,1].plot(hrs, data_i[0,:,INV_Q_AVG_IDX] / vabase, color='red', label='Reactive')
  #ax[0,1].set_ylabel('perunit')
  #ax[0,1].set_title ('Inverter Power at ' + inv_keys[0])
  #ax[0,1].legend(loc='best')

  #ax[0,1].plot(hrs, data_m[0,:,MTR_VOLTUNB_MAX_IDX], color='red', label='Max')
  #ax[0,1].set_ylabel('perunit')
  #ax[0,1].set_title ('Voltage Unbalance at ' + mtr_keys[0])

  avg1 = (data_h[:,:,HSE_AIR_AVG_IDX]).squeeze()
  avg2 = avg1.mean(axis=0)
  min1 = (data_h[:,:,HSE_AIR_MIN_IDX]).squeeze()
  min2 = min1.min(axis=0)
  max1 = (data_h[:,:,HSE_AIR_MAX_IDX]).squeeze()
  max2 = max1.max(axis=0)
  ax[0,1].plot(hrs, max2, color='blue', label='Max')
  ax[0,1].plot(hrs, min2, color='red', label='Min')
  ax[0,1].plot(hrs, avg2, color='green', label='Avg')
  ax[0,1].set_ylabel('degF')
  ax[0,1].set_title ('Temperature over All Houses')
  ax[0,1].legend(loc='best')

  if nBillingMeters > 0:
    vavg = (data_m[:,:,MTR_VOLT_AVG_IDX]).squeeze().mean(axis=0)
    vmin = (data_m[:,:,MTR_VOLT_MIN_IDX]).squeeze().min(axis=0)
    vmax = (data_m[:,:,MTR_VOLT_MAX_IDX]).squeeze().max(axis=0)
    ax[1,0].plot(hrs, vmax, color='blue', label='Max')
    ax[1,0].plot(hrs, vmin, color='red', label='Min')
    ax[1,0].plot(hrs, vavg, color='green', label='Avg')
    ax[1,0].set_xlabel('Hours')
    ax[1,0].set_ylabel('%')
    ax[1,0].set_title ('Voltage over all Meters')
    ax[1,0].legend(loc='best')
  else:
    ax[1,0].set_title ('No Billing Meter Voltages')

  ax[1,1].plot(hrs, data_h[0,:,HSE_AIR_AVG_IDX], color='blue', label='Mean')
  ax[1,1].plot(hrs, data_h[0,:,HSE_AIR_MIN_IDX], color='red', label='Min')
  ax[1,1].plot(hrs, data_h[0,:,HSE_AIR_MAX_IDX], color='green', label='Max')
  ax[1,1].plot(hrs, data_h[0,:,HSE_AIR_DEVC_IDX], color='magenta', label='DevC')
  ax[1,1].plot(hrs, data_h[0,:,HSE_AIR_DEVH_IDX], color='orange', label='DevH')
  ax[1,1].set_xlabel('Hours')
  ax[1,1].set_ylabel(HSE_AIR_AVG_UNITS)
  ax[1,1].set_title ('House Air at ' + hse_keys[0])
  ax[1,1].legend(loc='best')

  ax[0,2].plot(hrs, solar_kw, color='blue', label='Solar')
  ax[0,2].plot(hrs, battery_kw, color='red', label='Battery')
  ax[0,2].set_xlabel('Hours')
  ax[0,2].set_ylabel('kW')
  ax[0,2].set_title ('Total Inverter Power')
  ax[0,2].legend(loc='best')

  ax[1,2].plot(hrs, data_m[:,:,MTR_BILL_IDX].sum(axis=0), color='blue')
  ax[1,2].set_xlabel('Hours')
  ax[1,2].set_ylabel(MTR_BILL_UNITS)
  ax[1,2].set_title ('Total Meter Bill')

  if len(cap_keys) > 0 and bCollectedRegCapMetrics:
    ax[0,3].plot(hrs, data_c[:,:,CAP_COUNT_IDX].sum(axis=0), color='blue', label='Total')
    ax[0,3].set_ylabel('')
    ax[0,3].set_title ('Capacitor Switchings')
    ax[0,3].legend(loc='best')
  else:
    ax[0,3].set_title ('No Capacitors')

  if len(reg_keys) > 0 and bCollectedRegCapMetrics:
    ax[1,3].plot(hrs, data_r[:,:,REG_COUNT_IDX].sum(axis=0), color='blue', label='Total')
#   ax[1,3].plot(hrs, data_r[0,:,REG_COUNT_IDX], color='blue', label=reg_keys[0])
#   ax[1,3].plot(hrs, data_r[1,:,REG_COUNT_IDX], color='red', label=reg_keys[1])
#   ax[1,3].plot(hrs, data_r[2,:,REG_COUNT_IDX], color='green', label=reg_keys[2])
#   ax[1,3].plot(hrs, data_r[3,:,REG_COUNT_IDX], color='magenta', label=reg_keys[3])
    ax[1,3].set_xlabel('Hours')
    ax[1,3].set_ylabel('')
    ax[1,3].set_title ('Regulator Tap Changes')
    ax[1,3].legend(loc='best')
  else:
    ax[1,3].set_title ('No Regulators')

  ax[0,4].plot(hrs, (data_m[:,:,MTR_AHI_COUNT_IDX]).squeeze().sum(axis=0), color='blue', label='Range A Hi')
  ax[0,4].plot(hrs, (data_m[:,:,MTR_BHI_COUNT_IDX]).squeeze().sum(axis=0), color='cyan', label='Range B Hi')
  ax[0,4].plot(hrs, (data_m[:,:,MTR_ALO_COUNT_IDX]).squeeze().sum(axis=0), color='green', label='Range A Lo')
  ax[0,4].plot(hrs, (data_m[:,:,MTR_BLO_COUNT_IDX]).squeeze().sum(axis=0), color='magenta', label='Range B Lo')
  ax[0,4].plot(hrs, (data_m[:,:,MTR_OUT_COUNT_IDX]).squeeze().sum(axis=0), color='red', label='No Voltage')
  ax[0,4].set_ylabel('')
  ax[0,4].set_title ('Voltage Violation Counts')
  ax[0,4].legend(loc='best')

  ax[1,4].plot(hrs, (data_m[:,:,MTR_AHI_DURATION_IDX]).squeeze().sum(axis=0), color='blue', label='Range A Hi')
  ax[1,4].plot(hrs, (data_m[:,:,MTR_BHI_DURATION_IDX]).squeeze().sum(axis=0), color='cyan', label='Range B Hi')
  ax[1,4].plot(hrs, (data_m[:,:,MTR_ALO_DURATION_IDX]).squeeze().sum(axis=0), color='green', label='Range A Lo')
  ax[1,4].plot(hrs, (data_m[:,:,MTR_BLO_DURATION_IDX]).squeeze().sum(axis=0), color='magenta', label='Range B Lo')
  ax[1,4].plot(hrs, (data_m[:,:,MTR_OUT_DURATION_IDX]).squeeze().sum(axis=0), color='red', label='No Voltage')
  ax[1,3].set_xlabel('Hours')
  ax[1,4].set_ylabel('Seconds')
  ax[1,4].set_title ('Voltage Violation Durations')
  ax[1,4].legend(loc='best')

  plt.show()





#   Copyright (C) 2017-2019 Battelle Memorial Institute
# file: process_agents.py
"""Functions to plot data from GridLAB-D substation agents

Public Functions:
        :process_agents: Reads the data and metadata, then makes the plots.  

"""
import json;
import sys;
import numpy as np;
try:
  import matplotlib as mpl;
  import matplotlib.pyplot as plt;
except:
  pass

def process_agents(nameroot, dictname = '', want_dictionary=False):
    """ Plots cleared price, plus bids from the first HVAC controller

    This function reads *auction_nameroot_metrics.json* and  
    *controller_nameroot_metrics.json* for the data;
    it reads *nameroot_glm_dict.json* for the metadata. 
    These must all exist in the current working directory.  
    Makes one graph with 2 subplots:
    
    1. Cleared price from the only auction, and bid price from the first controller
    2. Bid quantity from the first controller

    Args:
        nameroot (str): name of the TESP case, not necessarily the same as the GLM case, without the extension
        dictname (str): metafile name (with json extension) for a different GLM dictionary, if it's not *nameroot_glm_dict.json*. Defaults to empty.
    """
    # first, read and print a dictionary of relevant agents
    if len (dictname) > 0:
        lp = open (dictname).read()
    else:
        lp = open (nameroot + '_agent_dict.json').read()
    dict = json.loads(lp)
    a_keys = list(dict['markets'].keys())
    a_keys.sort()
    c_keys = list(dict['controllers'].keys())
    c_keys.sort()
    if want_dictionary:
      print('\nMarket Dictionary:')
      print('ID Period Unit Init StDev')
      for key in a_keys:
          row = dict['markets'][key]
          print (key, row['period'], row['unit'], row['init_price'], row['init_stdev'])
      print('\nController Dictionary:')
      print('ID House Mode BaseDaylight Ramp Offset Cap')
      for key in c_keys:
          row = dict['controllers'][key]
          print (key, row['houseName'], row['control_mode'], row['daylight_set'], row['ramp'], row['offset_limit'], row['price_cap'])

    # read the auction metrics file
    lp_a = open ('auction_' + nameroot + '_metrics.json').read()
    lst_a = json.loads(lp_a)
    print ('\nAuction Metrics data starting', lst_a['StartTime'])

    # make a sorted list of the times, and NumPy array of times in hours
    lst_a.pop('StartTime')
    meta_a = lst_a.pop('Metadata')
    times = list(map(int,list(lst_a.keys())))
    times.sort()
    print ('There are', len (times), 'sample times at', times[1] - times[0], 'second intervals')
    hrs = np.array(times, dtype=np.float)
    denom = 3600.0
    hrs /= denom

    # parse the metadata for things of specific interest
    #print ('\nAuction Metadata [Variable Index Units]')
    for key, val in meta_a.items():
        #print (key, val['index'], val['units'])
        if key == 'clearing_price':
            CLEAR_PRICE_IDX = val['index']
            CLEAR_PRICE_UNITS = val['units']
        if key == 'clearing_type':
            CLEAR_TYPE_IDX = val['index']
            CLEAR_TYPE_UNITS = val['units']
        if key == 'consumer_surplus':
            CONSUMER_SURPLUS_IDX = val['index']
            CONSUMER_SURPLUS_UNITS = val['units']
        if key == 'average_consumer_surplus':
            AVERAGE_CONSUMER_SURPLUS_IDX = val['index']
            AVERAGE_CONSUMER_SURPLUS_UNITS = val['units']
        if key == 'supplier_surplus':
            SUPPLIER_SURPLUS_IDX = val['index']
            SUPPLIER_SURPLUS_UNITS = val['units']

    # create a NumPy array of all auction metrics
    data_a = np.empty(shape=(len(a_keys), len(times), len(lst_a[str(times[0])][a_keys[0]])), dtype=np.float)
    print ('\nConstructed', data_a.shape, 'NumPy array for Auctions')
    j = 0
    for key in a_keys:
        i = 0
        for t in times:
            ary = lst_a[str(t)][a_keys[j]]
            data_a[j, i,:] = ary
            i = i + 1
        j = j + 1

    # read the controller metrics file
    lp_c = open ('controller_' + nameroot + '_metrics.json').read()
    lst_c = json.loads(lp_c)
    print ('\nController Metrics data starting', lst_c['StartTime'])

    # parse the metadata for things of specific interest
    # c_keys = ['house1_R1_12_47_1_tm_507_thermostat_controller']
    lst_c.pop('StartTime')
    meta_c = lst_c.pop('Metadata')
    #print ('\nController Metadata [Variable Index Units]')
    for key, val in meta_c.items():
        #print (key, val['index'], val['units'])
        if key == 'bid_price':
            BID_P_IDX = val['index']
            BID_P_UNITS = val['units']
        elif key == 'bid_quantity':
            BID_Q_IDX = val['index']
            BID_Q_UNITS = val['units']

    # create a NumPy array of all controller metrics - many are 'missing' zero-bids
    data_c = np.empty(shape=(len(c_keys), len(times), len(meta_c.items())), dtype=np.float)
    print ('\nConstructed', data_c.shape, 'NumPy array for Controllers')
    zary = np.zeros(len(meta_c.items()))
    j = 0
    for key in c_keys:
        i = 0
        for t in times:
            if c_keys[j] in lst_c[str(t)]:
                ary = lst_c[str(t)][c_keys[j]]
            else:
                ary = zary
            data_c[j, i,:] = ary
            i = i + 1
        j = j + 1

    # identify the controller that put in the highest bid
    cidx = 0
    nbidding = 0
    max_p = 0.0
    for i in range (len(c_keys)):
      this_max_p = np.amax (data_c[i,:,BID_P_IDX])
      if this_max_p > 0.0:
        nbidding += 1
      if this_max_p > max_p:
        max_p = this_max_p
        cidx = i
    print ('Out of {:d} controllers, {:d} submitted bids and the highest bidder was {:s} [{:d}]'. format (len(c_keys), nbidding, c_keys[cidx], cidx))

    # display a plot
    fig, ax = plt.subplots(2,2, sharex = 'col')

    ax[0,0].plot(hrs, data_a[0,:,CLEAR_PRICE_IDX], color='blue', label='Cleared')
    ax[0,0].plot(hrs, data_c[cidx,:,BID_P_IDX], color='red', label='Bid')
    ax[0,0].set_ylabel(CLEAR_PRICE_UNITS)
    ax[0,0].set_title ('Prices at ' + c_keys[cidx])
    ax[0,0].legend(loc='best')

    ax[1,0].plot(hrs, data_c[cidx,:,BID_Q_IDX], color='red', marker='o', label='Quantity')
    ax[1,0].set_ylabel(BID_Q_UNITS)
    ax[1,0].set_title ('Bid Quantity at ' + c_keys[cidx])
    ax[1,0].set_xlabel('Hours')

    ax[0,1].plot(hrs, data_a[0,:,CONSUMER_SURPLUS_IDX].cumsum(), color='blue', label='Consumer')
    ax[0,1].plot(hrs, data_a[0,:,SUPPLIER_SURPLUS_IDX].cumsum(), color='red', label='Supplier')
    ax[0,1].set_ylabel(CONSUMER_SURPLUS_UNITS)
    ax[0,1].set_title ('Surplus')
    ax[0,1].legend(loc='best')

    q1 = (data_c[:,:,BID_Q_IDX]).squeeze()
    q2 = q1.sum(axis=0)
    ax[1,1].plot(hrs, q2, color='red')
    ax[1,1].set_ylabel(BID_Q_UNITS)
    ax[1,1].set_title ('Total Controller Bids')
    ax[1,1].set_xlabel('Hours')

    plt.show()




# -*- coding: utf-8 -*-
"""TMY3toCSV

Convert typical meteorological year version 3 (TMY3) data to
comma separated values (CSV) for common use by TESP agents.
"""
try:
    import matplotlib.pyplot as plt
except:
    pass
import dateutil
import io
try:
    from urllib2 import urlopen, Request
except ImportError:
    from urllib.request import urlopen, Request

import pandas as pd
import warnings
# DateOffset is not vectorized and will throw a nuisance warning
warnings.simplefilter(action='ignore', category=pd.errors.PerformanceWarning)

def readtmy3(filename=None, coerce_year=None, recolumn=True):
    '''
    Read a TMY3 file in to a pandas dataframe.
    '''

    if filename is None:
        try:
            filename = _interactive_load()
        except:
            raise Exception('Interactive load failed. Tkinter not supported '
                            'on this system. Try installing X-Quartz and '
                            'reloading')

    head = ['USAF', 'Name', 'State', 'TZ', 'latitude', 'longitude', 'altitude']

    if filename.startswith('http'):
        request = Request(filename, headers={'User-Agent':
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_5) '
            'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.87 '
            'Safari/537.36'})
        response = urlopen(request)
        csvdata = io.StringIO(response.read().decode(errors='ignore'))
    else:
        # assume it's accessible via the file system
        csvdata = open(filename, 'r')

    # read in file metadata, advance buffer to second line
    firstline = csvdata.readline()
    if 'Request Rejected' in firstline:
        raise IOError('Remote server rejected TMY file request')

    meta = dict(zip(head, firstline.rstrip('\n').split(",")))

    # convert metadata strings to numeric types
    meta['altitude'] = float(meta['altitude'])
    meta['latitude'] = float(meta['latitude'])
    meta['longitude'] = float(meta['longitude'])
    meta['TZ'] = float(meta['TZ'])
    meta['USAF'] = int(meta['USAF'])

    # use pandas to read the csv file/stringio buffer
    # header is actually the second line in file, but tell pandas to look for
    # header information on the 1st line (0 indexing) because we've already
    # advanced past the true first line with the readline call above.
    data = pd.read_csv(
        csvdata, header=0,
        parse_dates={'datetime': ['Date (MM/DD/YYYY)', 'Time (HH:MM)']},
        date_parser=lambda *x: _parsedate(*x, year=coerce_year),
        index_col='datetime')
    if recolumn:
        data = _recolumn(data)  # rename to standard column names

    data = data.tz_localize(int(meta['TZ']*3600))
    return data, meta


def _interactive_load():
    import Tkinter
    from tkFileDialog import askopenfilename
    Tkinter.Tk().withdraw()  # Start interactive file input
    return askopenfilename()


def _parsedate(ymd, hour, year=None):
    # stupidly complicated due to TMY3's usage of hour 24
    # and dateutil's inability to handle that.
    offset_hour = int(hour[:2]) - 1
    offset_datetime = '{} {}:00'.format(ymd, offset_hour)
    offset_date = dateutil.parser.parse(offset_datetime)
    true_date = offset_date + dateutil.relativedelta.relativedelta(hours=1)
    if year is not None:
        true_date = true_date.replace(year=year)
    return true_date


def _recolumn(tmy3_dataframe):
    """
    Rename the columns of the TMY3 DataFrame.

    Parameters
    ----------
    tmy3_dataframe : DataFrame
    inplace : bool
        passed to DataFrame.rename()

    Returns
    -------
    Recolumned DataFrame.
    """
    raw_columns = 'ETR (W/m^2),ETRN (W/m^2),GHI (W/m^2),GHI source,GHI uncert (%),DNI (W/m^2),DNI source,DNI uncert (%),DHI (W/m^2),DHI source,DHI uncert (%),GH illum (lx),GH illum source,Global illum uncert (%),DN illum (lx),DN illum source,DN illum uncert (%),DH illum (lx),DH illum source,DH illum uncert (%),Zenith lum (cd/m^2),Zenith lum source,Zenith lum uncert (%),TotCld (tenths),TotCld source,TotCld uncert (code),OpqCld (tenths),OpqCld source,OpqCld uncert (code),Dry-bulb (C),Dry-bulb source,Dry-bulb uncert (code),Dew-point (C),Dew-point source,Dew-point uncert (code),RHum (%),RHum source,RHum uncert (code),Pressure (mbar),Pressure source,Pressure uncert (code),Wdir (degrees),Wdir source,Wdir uncert (code),Wspd (m/s),Wspd source,Wspd uncert (code),Hvis (m),Hvis source,Hvis uncert (code),CeilHgt (m),CeilHgt source,CeilHgt uncert (code),Pwat (cm),Pwat source,Pwat uncert (code),AOD (unitless),AOD source,AOD uncert (code),Alb (unitless),Alb source,Alb uncert (code),Lprecip depth (mm),Lprecip quantity (hr),Lprecip source,Lprecip uncert (code),PresWth (METAR code),PresWth source,PresWth uncert (code)'

    new_columns = [
        'ETR', 'ETRN', 'GHI', 'GHISource', 'GHIUncertainty',
        'DNI', 'DNISource', 'DNIUncertainty', 'DHI', 'DHISource',
        'DHIUncertainty', 'GHillum', 'GHillumSource', 'GHillumUncertainty',
        'DNillum', 'DNillumSource', 'DNillumUncertainty', 'DHillum',
        'DHillumSource', 'DHillumUncertainty', 'Zenithlum',
        'ZenithlumSource', 'ZenithlumUncertainty', 'TotCld', 'TotCldSource',
        'TotCldUnertainty', 'OpqCld', 'OpqCldSource', 'OpqCldUncertainty',
        'DryBulb', 'DryBulbSource', 'DryBulbUncertainty', 'DewPoint',
        'DewPointSource', 'DewPointUncertainty', 'RHum', 'RHumSource',
        'RHumUncertainty', 'Pressure', 'PressureSource',
        'PressureUncertainty', 'Wdir', 'WdirSource', 'WdirUncertainty',
        'Wspd', 'WspdSource', 'WspdUncertainty', 'Hvis', 'HvisSource',
        'HvisUncertainty', 'CeilHgt', 'CeilHgtSource', 'CeilHgtUncertainty',
        'Pwat', 'PwatSource', 'PwatUncertainty', 'AOD', 'AODSource',
        'AODUncertainty', 'Alb', 'AlbSource', 'AlbUncertainty',
        'Lprecipdepth', 'Lprecipquantity', 'LprecipSource',
        'LprecipUncertainty', 'PresWth', 'PresWthSource',
        'PresWthUncertainty']

    mapping = dict(zip(raw_columns.split(','), new_columns))
    return tmy3_dataframe.rename(columns=mapping)


def weathercsv(tmyfile,outputfile,start_time,end_time,year_):
    """Converts TMY3 weather data to CSV in the requested time range

    :param tmyfile: string
        the input TMY3 data file
    :param outputfile: string
        the output CSV data file
    :param start_time: string
        the starting date and time of interest, like `2013-07-01 00:00:00`
    :param end_time: string
        the ending date and time of interest, like `2013-07-08 00:00:00`
    :param year: integer
        the year of interest
    :return: nothing
    """
    global dts
    global result2
    tmydata=readtmy3(tmyfile)
    print ('  latitude', tmydata[1]['latitude'])
    print ('  longitude', tmydata[1]['longitude'])
    print ('  altitude', tmydata[1]['altitude'], 'm')
    print ('  TZmeridian', 15.0 * tmydata[1]['TZ'])
    
    temperature=tmydata[0]['DryBulb']*1.8 + 32.0 # convert to Fahrenheit
    humidity=tmydata[0]['RHum']/100.0            # convert to per-unit
    solar_direct=tmydata[0].DNI*0.09290304       # convert to W/sq foot
    solar_diffuse=tmydata[0].DHI*0.09290304      # convert to W/sq foot
    pressure=tmydata[0]['Pressure']              # stays in mbar
    wind_speed=tmydata[0]['Wspd']*3600.0/1609.0  # convert to mph
    result=pd.concat([temperature,humidity,solar_direct,solar_diffuse,pressure,wind_speed], 
                     axis=1, 
                     keys=['temperature','humidity','solar_direct','solar_diffuse','pressure','wind_speed'])
    result.index=result.index.tz_localize(None)  # remove the time zone
   
    #9.12 is a cloudy day
    
    result.index = result.index + pd.DateOffset(year=year_)  # change all the year index to a same one
    result2 = result.resample(rule='5Min',closed='left').first()
    result2=result2.interpolate(method='quadratic')
    # get the weather data for give time period and remove the inproper value generated by interpolation 
    dts=result2.loc[start_time:end_time,:].copy()
    dts.solar_direct[dts.solar_direct<0]=0
    dts.solar_direct[dts.solar_direct<1e-4]=0
    dts.solar_diffuse[dts.solar_diffuse<0]=0
    dts.solar_diffuse[dts.solar_diffuse<1e-4]=0
    dts.wind_speed[dts.wind_speed<0]=0
    dts.wind_speed[dts.wind_speed<1e-4]=0
    dts.to_csv(outputfile, float_format = '%.4f')
    
def weathercsv_cloudy_day(start_time,end_time,outputfile):
    day_weather=result2.loc[start_time:end_time,:]
    a=day_weather.temperature
    b=a.tolist()
    #define start and end time
    start_down=int(13*12); stop_down=int(13.125*12)
    start_up=int(14.5*12); stop_up=int(16*12)
    # define a down magnitude
    mag=3
    slope_down=-mag/(stop_down-start_down); slope_up=mag/(stop_up-start_up)
    
    for i in range(len(b)):
        if i >= start_down and i <= stop_down:
            b[i] = b[i] + slope_down*(i-start_down)
        elif (i > stop_down) and (i < start_up):
            b[i] = b[i] - mag
        elif (i >= start_up) and (i <= stop_up):
            b[i] =b[i] - mag + slope_up*(i-start_up)
    plt.subplot(211)
    plt.plot(b)
            
    c=day_weather.solar_direct
    d=c.tolist()
    # the solar reduced factor
    solarR=0.9
    start_mag=d[start_down]
    stop_mag=(1-solarR)*d[stop_down]
    start_up_mag=(1-solarR)*d[start_up]
    slope_down=-(start_mag-stop_mag)/(stop_down-start_down); slope_up=(d[stop_up]-start_up_mag)/(stop_up-start_up)    
    for i in range(len(d)):
        if i >= start_down and i <= stop_down:
            d[i] = d[i] + slope_down*(i-start_down)
        elif (i > stop_down) and (i < start_up):
            d[i] = 0.1*d[i]
        elif (i >= start_up) and (i <= stop_up):
            d[i] =start_up_mag + slope_up*(i-start_up)
    plt.subplot(212)
    plt.plot(d,color='r')    
    day_weather['temperature']=b
    day_weather['solar_direct']=d
    day_weather.to_csv(outputfile)

def _tests():
    #create a csv file contain the weather data for the input time period from the input tmy3 file
    weathercsv('../../support/weather/TX-Houston_Bush_Intercontinental.tmy3','weather.csv','2000-01-01 00:00:00','2000-01-14 00:00:00',2000)
    #create a csv file for a cloudy day for the selected date 
    weathercsv_cloudy_day('2000-01-01 00:00:00','2000-01-02 00:00:00','cloudy_day.csv')

if __name__ == '__main__':
    _tests()
